Module netsblox_vm::bytecode
source · Expand description
Tools for generating executable ByteCode from a project’s abstract syntax tree.
To generate bytecode from a project, you can use ByteCode::compile.
Structs
- An interpreter-ready sequence of instructions.
- Entity-level
ScriptInfoin aByteCodeobject. - A structure that stores the initialization state of a project.
- Location lookup table from bytecode address to original AST location.
- Script information in a
ByteCodeobject.
Enums
- A NetsBlox project compile error generated by
ByteCode::compile.