Module netsblox_vm::cli
source · Expand description
Access to the standard netsblox-vm CLI.
Access to this submodule requires the cli feature flag, which is enabled by default.
This submodule acts as a full-fledged implementation of a usable netsblox-vm CLI front-end.
This includes being able to compile and run individual project files locally,
as well as a server mode where a user can connect to the server from the browser
and use the block-based interface to write, upload, and run code on the server.
Note that server mode does not yet support multiple simultaneous.
Enums
- Standard NetsBlox VM project actions that can be performed
Functions
- Runs a CLI client using the given
Modeconfiguration.