pub struct Process<'gc, C: CustomTypes<S>, S: System<C>> {
    pub global_context: Gc<'gc, RefLock<GlobalContext<'gc, C, S>>>,
    pub state: C::ProcessState,
    /* private fields */
}
A ByteCode execution primitive.

A Process is a self-contained thread of execution. It maintains its own state machine for executing instructions step by step.

Fields

global_context: Gc<'gc, RefLock<GlobalContext<'gc, C, S>>>state: C::ProcessState

Implementations

source

impl<'gc, C: CustomTypes<S>, S: System<C>> Process<'gc, C, S>

source

pub fn new(context: ProcContext<'gc, C, S>) -> Self

Creates a new Process with the given starting context.

source

pub fn is_running(&self) -> bool

Checks if the process is currently running. Note that the process will not run on its own (see Process::step).

source

pub fn current_entity(&self) -> Gc<'gc, RefLock<Entity<'gc, C, S>>>

Gets the current active entity of this process. Note that this is not necessarily the same entity that was used to construct the process, as blocks such as “tell _ to _” and “ask _ for _” can change the context entity.

source

pub fn step( &mut self, mc: &Mutation<'gc> ) -> Result<ProcessStep<'gc, C, S>, ExecError<C, S>>

Executes a single bytecode instruction. The return value can be used to determine what additional effects the script has requested, as well as to retrieve the return value or execution error in the event that the process terminates.

The process transitions to the idle state (see Process::is_running) upon failing with Err or succeeding with ProcessStep::Terminate.