Module netsblox_vm::runtime

source ·
Miscellaneous types representing runtime state.

Modules

Structs

Enums

Traits

  • A collection of static settings for using custom native types.
  • Gets the type of value that is stored.
  • Represents all the features of an implementing system.

Type Aliases

  • The type used to represent numbers in the runtime.