Enum netsblox_vm::runtime::ToJsonError
source · pub enum ToJsonError<C: CustomTypes<S>, S: System<C>> {
BadNumber(f64),
ComplexType(Type<C, S>),
Cyclic,
}Variants
BadNumber(f64)
The value was or contained a number which could not be expressed as f64.
ComplexType(Type<C, S>)
The value was or contained a type that cannot be exported as primitive Json.
Cyclic
The value contained a cycle, which Json forbids.