Type Definition netsblox_vm::runtime::Number
source · pub type Number = CheckedFloat<f64, NumberChecker>;Expand description
The type used to represent numbers in the runtime.
pub type Number = CheckedFloat<f64, NumberChecker>;The type used to represent numbers in the runtime.