Struct netsblox_vm::slotmap::SlotMap
source · pub struct SlotMap<K: Key, T> { /* private fields */ }Expand description
A dense, resizable array that supports generational indexing.
You can use the [new_key] macro to create a new key type to use.
It is recommended to use different key types for different collections to avoid accidentally using a key from a different map.
Implementations
sourceimpl<K: Key, T> SlotMap<K, T>
impl<K: Key, T> SlotMap<K, T>
sourcepub fn insert(&mut self, value: T) -> K
pub fn insert(&mut self, value: T) -> K
Adds a new value to the map and returns a new key that references it.
sourcepub fn remove(&mut self, key: K) -> Option<T>
pub fn remove(&mut self, key: K) -> Option<T>
Removes a value from the map and returns it (if it existed).
It is guaranteed that all future accesses with the removed key will return None.
sourcepub fn get_mut(&mut self, key: K) -> Option<&mut T>
pub fn get_mut(&mut self, key: K) -> Option<&mut T>
Get a mutable reference to a value in the map.