pub struct Clock { /* private fields */ }
A clock with optional coarse granularity.

Implementations

source

impl Clock

source

pub fn new(utc_offset: UtcOffset, cache_precision: Option<Precision>) -> Self

Creates a new Clock with the specified UtcOffset and (optional) cache Precision (see Clock::read).

source

pub fn read(&self, precision: Precision) -> OffsetDateTime

Reads the current time with the specified level of precision. If caching was enabled by Clock::new, requests at or below the cache precision level will use the cached timestamp. In any other case, the real current time is fetched by Clock::update and the result is stored in the cache if caching is enabled.

source

pub fn update(&self) -> OffsetDateTime

Reads the real world time and stores the result in the cache if caching was enabled by Clock::new.