pub enum SyscallMenu {
    Entry {
        label: String,
        value: String,
    },
    Submenu {
        label: String,
        content: Vec<SyscallMenu>,
    },
}
An entry to display in the syscall dropdown when running in server mode.

A single syscall can be listed multiple times, e.g., under different submenu categorizations. These are not checked against the syscalls actually supported by your runtime. You are responsible for implementing syscalls and ensuring they are accurately shown in the menu if desired.

Variants



Entry

Fields

label: String
value: String

A syscall name.



Submenu

Fields

label: String
content: Vec<SyscallMenu>

A labeled submenu of syscalls.

Implementations

source

impl SyscallMenu

source

pub fn simple_entry(value: String) -> Self

Creates an instance of SyscallMenu::Entry with the given string as both the label and value.