Struct netsblox_vm::template::ExtensionArgs
source · pub struct ExtensionArgs<'a> {
pub server: &'a str,
pub syscalls: &'a [SyscallMenu],
pub omitted_elements: &'a [&'a str],
pub pull_interval: Duration,
}Expand description
Arguments used to construct a templated extension.
Fields
server: &'a strThe NetsBlox VM server to connect to.
syscalls: &'a [SyscallMenu]The syscall menu structure to generate for syscall blocks.
omitted_elements: &'a [&'a str]A list of XML element names to omit from the XML sent to the VM server.
pull_interval: DurationThe duration between successive calls to pull status from the VM server.
Implementations