Enum netsblox_vm::cli::Mode
source · pub enum Mode {
Run {
src: String,
role: Option<String>,
server: String,
},
Dump {
src: String,
role: Option<String>,
},
Start {
server: String,
addr: String,
port: u16,
},
}Expand description
Standard NetsBlox VM project actions that can be performed
Variants
Run
Fields
Compiles and runs a single project file
Dump
Fields
Compiles a single project file and dumps its disassembly to stdout
Start
Fields
Starts an execution server which you can connect to from the browser