Enum netsblox_vm::cli::Mode

source ·
pub enum Mode {
    Run {
        src: String,
        role: Option<String>,
        server: String,
    },
    Dump {
        src: String,
        role: Option<String>,
    },
    Start {
        server: String,
        addr: String,
        port: u16,
    },
}
Standard NetsBlox VM project actions that can be performed

Variants



Run

Fields

src: String

Path to the (xml) project file

role: Option<String>

The specific role to run, or none if not ambiguous

server: String

Address of the NetsBlox server (default: https://editor.netsblox.org)

Compiles and runs a single project file



Dump

Fields

src: String

Path to the (xml) project file

role: Option<String>

The specific role to compile, or none if not ambiguous

Compiles a single project file and dumps its disassembly to stdout



Start

Fields

server: String

Address of the NetsBlox server (default: https://editor.netsblox.org)

addr: String

The address of this machine, which others use to send HTTP requests (default 127.0.0.1)

port: u16

The port to bind for the web server (default 6286)

Starts an execution server which you can connect to from the browser