Module netsblox_vm::runtime
source · Modules
- Special types for working with a
SymbolTable.
Structs
- A blocking handle for a
BarrierCondition. - Waits for the destruction of all associated
Barrierhandles. - Information about a closure/lambda function.
- A collection of implementation options that could be used for implementing a customizable
System. - A type conversion error on a
Value. - A collection of graphical effects related to an entity
- Information about an entity (sprite or stage).
- Global information about the execution state of an entire project.
- A value representing the identity of a
Value. FloatCheckertype used for validating aNumber.- A collection of properties related to an entity.
- Settings to use for a
Process. - Holds a collection of variables in an execution context.
- Represents a variable whose value is being monitored.
Enums
- The result of a successful call to an async poller operation such as in
System. - A non-value-returning command issued from the runtime.
- The status of a potentially-handled command.
- The kind of entity being represented.
- The cause/explanation of an execution error.
- The error promotion paradigm to use for certain types of runtime errors.
- An event type which can be set to trigger the execution of a script.
- Types of
Systemresources, grouped into feature categories. - A key from the keyboard.
- The result of an operation that might be synchronous or asynchronous.
- Error type used by
NumberChecker. - A value-returning request issued from the runtime.
- The status of a potentially-handled request.
- Represents a shared mutable resource.
- The time as determined by an implementation of
System. - The type of a
Value. - Any primitive value.
Traits
- A collection of static settings for using custom native types.
- Gets the type of value that is stored.
- The required interface used by
CustomTypes::Intermediate. - Represents all the features of an implementing system.
Type Definitions
- The type used to represent numbers in the runtime.