Enum netsblox_vm::template::SyscallMenu
source · pub enum SyscallMenu {
Entry {
label: String,
value: String,
},
Submenu {
label: String,
content: Vec<SyscallMenu>,
},
}Expand description
An entry to display in the syscall dropdown when running in server mode.
A single syscall can be listed multiple times, e.g., under different submenu categorizations. These are not checked against the syscalls actually supported by your runtime. You are responsible for implementing syscalls and ensuring they are accurately shown in the menu if desired.
Variants
Implementations
sourceimpl SyscallMenu
impl SyscallMenu
sourcepub fn simple_entry(value: String) -> Self
pub fn simple_entry(value: String) -> Self
Creates an instance of SyscallMenu::Entry with the given string as both the label and value.